package jrummikub.view.impl;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.image.BufferedImage;
import java.util.Collection;
import java.util.Collections;

import javax.swing.ImageIcon;
import javax.swing.border.MatteBorder;

import jrummikub.model.Position;
import jrummikub.model.Stone;
import jrummikub.util.Pair;
import jrummikub.view.IHandPanel;

/**
 * Implementation of the board
 */
@SuppressWarnings("serial")
class HandPanel extends AbstractStonePanel implements IHandPanel {
	private final static int HAND_HEIGHT = 2;
	private final static int HAND_WIDTH = 14;

	private final static BufferedImage BACKGROUND;
	static {
		ImageIcon image = new ImageIcon(
				HandPanel.class.getResource("/jrummikub/resource/wood.png"));
		BACKGROUND = new BufferedImage(image.getIconWidth(), image.getIconHeight(),
				BufferedImage.TYPE_INT_RGB);

		image.paintIcon(null, BACKGROUND.createGraphics(), 0, 0);
	}
	private BufferedImage scaledBackground = BACKGROUND;

	private boolean repaintAll = true;

	private Collection<Stone> selectedStones = Collections.emptyList();

	/**
	 * Creates a new Board instance
	 */
	HandPanel() {
		setBorder(new MatteBorder(0, 1, 0, 1, Color.DARK_GRAY));

		addComponentListener(new ComponentAdapter() {

			@Override
			public void componentResized(ComponentEvent e) {
				Insets insets = getInsets();
				int size = (getHeight() - insets.top - insets.bottom) / HAND_HEIGHT;

				getStonePainter().setScale(size * StonePainter.HEIGHT_SCALE);

				setSize(new Dimension(HAND_WIDTH * getStonePainter().getStoneWidth()
						+ insets.left + insets.right, getHeight()));

				repaintAll = true;
			}
		});
	}

	private BufferedImage getScaledBackground(int size) {
		BufferedImage scaled = new BufferedImage(size, size,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D g = scaled.createGraphics();
		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
				RenderingHints.VALUE_INTERPOLATION_BICUBIC);

		g.drawImage(BACKGROUND, 0, 0, size, size, null);

		return scaled;
	}

	@Override
	protected void paintComponent(Graphics g1) {
		Insets insets = getInsets();
		int x = insets.left, y = insets.top, width = getWidth() - insets.left
				- insets.right, height = getHeight() - insets.top - insets.bottom;
		Graphics2D g = (Graphics2D) g1.create(x, y, width, height);

		int size = height / HAND_HEIGHT;

		if (repaintAll) {
			if (scaledBackground.getHeight() != size)
				scaledBackground = getScaledBackground(size);

			for (int i = 0; i < HAND_HEIGHT; ++i) {
				for (int xpos = -size * i / 3; xpos < width; xpos += size) {
					g.drawImage(scaledBackground, xpos, size * i, null);
				}
			}
		}

		for (Pair<Stone, Position> entry : getStones()) {
			getStonePainter().paintStone(g, entry.getFirst(), entry.getSecond(),
					selectedStones.contains(entry.getFirst()), entry.getFirst() == getHoveredStone());
		}
	}

	@Override
	public void setStones(Iterable<Pair<Stone, Position>> stones) {
		super.setStones(stones);
		repaintAll = true;
		repaint();
	}

	/**
	 * Sets the stones that are to be painted selected
	 * 
	 * @param stones
	 *          the selected stones
	 */
	void setSelectedStones(Collection<Stone> stones) {
		selectedStones = stones;
		repaint();
	}
}
